@@ -1,13 +1,13 @@
 #!/bin/bash
 # Copyright 1999-2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: ebuild.sh 9196 2008-01-12 01:47:04Z zmedico $
+# $Id: ebuild.sh 10330 2008-05-13 23:09:03Z zmedico $
 
 PORTAGE_BIN_PATH="${PORTAGE_BIN_PATH:-/usr/lib/portage/bin}"
 PORTAGE_PYM_PATH="${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}"
 
 SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/dev/random"
-export SANDBOX_PREDICT="${SANDBOX_PREDICT}:${PORTAGE_PYM_PATH}:${PORTAGE_DEPCACHEDIR}"
+export SANDBOX_PREDICT
 export SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:/dev/stdout:/dev/stderr:${PORTAGE_TMPDIR}"
 export SANDBOX_READ="${SANDBOX_READ}:/:/dev/shm:/dev/stdin:${PORTAGE_TMPDIR}"
 # Don't use sandbox's BASH_ENV for new shells because it does
@@ -33,22 +33,26 @@ fi
 
 qa_source() {
 	local shopts=$(shopt) OLDIFS="$IFS"
-	source "$@" || return 1
+	local retval
+	source "$@"
+	retval=$?
 	[[ $shopts != $(shopt) ]] &&
 		eqawarn "QA Notice: Global shell options changed and were not restored while sourcing '$*'"
 	[[ "$IFS" != "$OLDIFS" ]] &&
 		eqawarn "QA Notice: Global IFS changed and was not restored while sourcing '$*'"
-	return 0
+	return $retval
 }
 
 qa_call() {
 	local shopts=$(shopt) OLDIFS="$IFS"
-	"$@" || return 1
+	local retval
+	"$@"
+	retval=$?
 	[[ $shopts != $(shopt) ]] &&
 		eqawarn "QA Notice: Global shell options changed and were not restored while calling '$*'"
 	[[ "$IFS" != "$OLDIFS" ]] &&
 		eqawarn "QA Notice: Global IFS changed and was not restored while calling '$*'"
-	return 0
+	return $retval
 }
 
 # subshell die support
@@ -144,7 +148,7 @@ useq() {
 		[[ ${EMERGE_FROM} != binary ]] ; then
 		# TODO: Implement PORTAGE_IUSE for binary packages. Currently,
 		# it is only valid for build time phases.
-		echo "${u}" | egrep -q "${PORTAGE_IUSE}" || \
+		[[ $u =~ $PORTAGE_IUSE ]] || \
 			eqawarn "QA Notice: USE Flag '${u}' not" \
 				"in IUSE for ${CATEGORY}/${PF}"
 	fi
@@ -368,9 +372,22 @@ unpack() {
 			LHa|LHA|lha|lzh)
 				lha xfq "${srcdir}${x}" || die "$myfail"
 				;;
-			a|deb)
+			a)
 				ar x "${srcdir}${x}" || die "$myfail"
 				;;
+			deb)
+				# Unpacking .deb archives can not always be done with
+				# `ar`.  For instance on AIX this doesn't work out.  If
+				# we have `deb2targz` installed, prefer it over `ar` for
+				# that reason.  We just make sure on AIX `deb2targz` is
+				# installed.
+				if type -P deb2targz > /dev/null; then
+					deb2targz "${srcdir}/${x}" || die "$myfail"
+					mv "${srcdir}/${x/.deb/.tar.gz}" data.tar.gz
+				else
+					ar x "${srcdir}/${x}" || die "$myfail"
+				fi
+				;;
 			lzma)
 				if [ "${y}" == "tar" ]; then
 					lzma -dc "${srcdir}${x}" | tar xof - ${tar_opts}
@@ -645,12 +662,11 @@ dyn_unpack() {
 	fi
 	if [ "${newstuff}" == "yes" ]; then
 		# We don't necessarily have privileges to do a full dyn_clean here.
+		rm -rf "${PORTAGE_BUILDDIR}"/{.unpacked,.compiled,.tested,.packaged,build-info}
 		rm -rf "${WORKDIR}"
-		if [ -d "${T}" ] && ! hasq keeptemp ${FEATURES} ; then
+		if [ -d "${T}" ] && \
+			! hasq keeptemp $FEATURES && ! hasq keepwork $FEATURES ; then
 			rm -rf "${T}" && mkdir "${T}"
-		else
-			[ -e "${T}/environment" ] && \
-				mv "${T}/environment" "${T}/environment.keeptemp"
 		fi
 	fi
 	if [ -e "${WORKDIR}" ]; then
@@ -687,11 +703,10 @@ dyn_clean() {
 	fi
 
 	rm -rf "${PORTAGE_BUILDDIR}/image" "${PORTAGE_BUILDDIR}/homedir"
+	rm -f "${PORTAGE_BUILDDIR}/.installed"
 
-	if ! hasq keeptemp $FEATURES; then
+	if ! hasq keeptemp $FEATURES && ! hasq keepwork $FEATURES ; then
 		rm -rf "${T}"
-	else
-		[ -e "${T}/environment" ] && mv "${T}/environment" "${T}/environment.keeptemp"
 	fi
 
 	if ! hasq keepwork $FEATURES; then
@@ -700,7 +715,6 @@ dyn_clean() {
 		rm -rf "${PORTAGE_BUILDDIR}/.unpacked"
 		rm -rf "${PORTAGE_BUILDDIR}/.compiled"
 		rm -rf "${PORTAGE_BUILDDIR}/.tested"
-		rm -rf "${PORTAGE_BUILDDIR}/.installed"
 		rm -rf "${PORTAGE_BUILDDIR}/.packaged"
 		rm -rf "${PORTAGE_BUILDDIR}/build-info"
 		rm -rf "${WORKDIR}"
@@ -962,6 +976,15 @@ dyn_install() {
 	#some packages uses an alternative to $S to build in, cause
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
+
+	# Reset exeinto(), docinto(), insinto(), and into() state variables
+	# in case the user is running the install phase multiple times
+	# consecutively via the ebuild command.
+	export DESTTREE=/usr
+	export INSDESTTREE=""
+	export _E_EXEDESTTREE_=""
+	export _E_DOCDESTTREE_=""
+
 	ebuild_phase src_install
 	touch "${PORTAGE_BUILDDIR}/.installed"
 	vecho ">>> Completed installing ${PF} into ${D}"
@@ -1030,7 +1053,7 @@ dyn_help() {
 	echo "  help        : show this help screen"
 	echo "  setup       : execute package specific setup actions"
 	echo "  fetch       : download source archive(s) and patches"
-	echo "  digest      : create a digest and a manifest file for the package"
+	echo "  digest      : create a manifest file for the package"
 	echo "  manifest    : create a manifest file for the package"
 	echo "  unpack      : unpack/patch sources (auto-fetch if needed)"
 	echo "  compile     : compile sources (auto-fetch/unpack if needed)"
@@ -1093,7 +1116,7 @@ debug-print() {
 
 		# extra user-configurable targets
 		if [ "$ECLASS_DEBUG_OUTPUT" == "on" ]; then
-			echo "debug: $1"
+			echo "debug: $1" >&2
 		elif [ -n "$ECLASS_DEBUG_OUTPUT" ]; then
 			echo "debug: $1" >> $ECLASS_DEBUG_OUTPUT
 		fi
@@ -1380,8 +1403,13 @@ PORTAGE_MUTABLE_FILTERED_VARS="AA HOSTNAME"
 
 # @FUNCTION: filter_readonly_variables
 # @DESCRIPTION: [--filter-sandbox] [--allow-extra-vars]
-# Read an environment from stdin and echo to stdout while filtering readonly
-# variables.
+# Read an environment from stdin and echo to stdout while filtering variables
+# with names that are known to cause interference:
+#
+#   * some specific variables for which bash does not allow assignment
+#   * some specific variables that affect portage or sandbox behavior
+#   * variable names that begin with a digit or that contain any
+#     non-alphanumeric characters that are not be supported by bash
 #
 # --filter-sandbox causes all SANDBOX_* variables to be filtered, which
 # is only desired in certain cases, such as during preprocessing or when
@@ -1403,16 +1431,16 @@ PORTAGE_MUTABLE_FILTERED_VARS="AA HOSTNAME"
 # builtin command. To avoid this problem, this function filters those
 # variables out and discards them. See bug #190128.
 filter_readonly_variables() {
-	local x filtered_vars var_grep
+	local x filtered_vars
 	local readonly_bash_vars="DIRSTACK EUID FUNCNAME GROUPS
 		PIPESTATUS PPID SHELLOPTS UID"
 	local filtered_sandbox_vars="SANDBOX_ACTIVE SANDBOX_BASHRC
 		SANDBOX_DEBUG_LOG SANDBOX_DISABLED SANDBOX_LIB
 		SANDBOX_LOG SANDBOX_ON"
 	filtered_vars="${readonly_bash_vars} ${READONLY_PORTAGE_VARS}
-		BASH_[_[:alnum:]]* PATH"
+		BASH_.* PATH"
 	if hasq --filter-sandbox $* ; then
-		filtered_vars="${filtered_vars} SANDBOX_[_[:alnum:]]*"
+		filtered_vars="${filtered_vars} SANDBOX_.*"
 	else
 		filtered_vars="${filtered_vars} ${filtered_sandbox_vars}"
 	fi
@@ -1426,20 +1454,8 @@ filter_readonly_variables() {
 			${PORTAGE_MUTABLE_FILTERED_VARS}
 		"
 	fi
-	set -f
-	for x in ${filtered_vars} ; do
-		var_grep="${var_grep}|${x}"
-	done
-	set +f
-	var_grep=${var_grep:1} # strip the first |
-	var_grep="(^|^declare[[:space:]]+-[^[:space:]]+[[:space:]]+|^export[[:space:]]+)(${var_grep})=.*"
-	# The sed is to remove the readonly attribute from variables such as those
-	# listed in READONLY_EBUILD_METADATA, since having any readonly attributes
-	# persisting in the saved environment can be inconvenient when it
-	# eventually needs to be reloaded.
-	"${PORTAGE_BIN_PATH}"/filter-bash-environment.py "${var_grep}" | sed -r \
-		-e 's:^declare[[:space:]]+-r[[:space:]]+:declare :' \
-		-e 's:^declare[[:space:]]+-([[:alnum:]]*)r([[:alnum:]]*)[[:space:]]+:declare -\1\2 :'
+
+	"${PORTAGE_BIN_PATH}"/filter-bash-environment.py "${filtered_vars}"
 }
 
 # @FUNCTION: preprocess_ebuild_env
@@ -1561,14 +1577,6 @@ if [ -n "${EBUILD_SH_ARGS}" ] && \
 #		source ${X} || die "Failed to source ${X}"
 #	done
 
-else
-
-killparent() {
-	trap INT
-	kill ${PORTAGE_MASTER_PID}
-}
-trap "killparent" INT
-
 fi # "$*"!="depend" && "$*"!="clean" && "$*" != "setup"
 
 export SANDBOX_ON="1"
@@ -1579,9 +1587,17 @@ unset E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
 # Turn of extended glob matching so that g++ doesn't get incorrectly matched.
 shopt -u extglob
 
-QA_INTERCEPTORS="javac java-config python python-config perl grep egrep fgrep sed gcc g++ cc bash awk nawk gawk pkg-config"
+if [[ ${EBUILD_PHASE} == depend ]] ; then
+	QA_INTERCEPTORS="awk bash cc egrep fgrep g++
+		gawk gcc grep javac java-config nawk perl
+		pkg-config python python-config sed"
+elif [[ ${EBUILD_PHASE} == clean* ]] ; then
+	unset QA_INTERCEPTORS
+else
+	QA_INTERCEPTORS="autoconf automake aclocal libtoolize"
+fi
 # level the QA interceptors if we're in depend
-if hasq "depend" "${EBUILD_SH_ARGS}"; then
+if [[ -n ${QA_INTERCEPTORS} ]] ; then
 	for BIN in ${QA_INTERCEPTORS}; do
 		BIN_PATH=$(type -Pf ${BIN})
 		if [ "$?" != "0" ]; then
@@ -1589,14 +1605,32 @@ if hasq "depend" "${EBUILD_SH_ARGS}"; then
 		else
 			BODY="${BIN_PATH} \"\$@\"; return \$?"
 		fi
-		FUNC_SRC="${BIN}() {
-		if [ \$ECLASS_DEPTH -gt 0 ]; then
-			eqawarn \"QA Notice: '${BIN}' called in global scope: eclass \${ECLASS}\"
+		if [[ ${EBUILD_PHASE} == depend ]] ; then
+			FUNC_SRC="${BIN}() {
+				if [ \$ECLASS_DEPTH -gt 0 ]; then
+					eqawarn \"QA Notice: '${BIN}' called in global scope: eclass \${ECLASS}\"
+				else
+					eqawarn \"QA Notice: '${BIN}' called in global scope: \${CATEGORY}/\${PF}\"
+				fi
+			${BODY}
+			}"
+		elif hasq ${BIN} autoconf automake aclocal libtoolize ; then
+			FUNC_SRC="${BIN}() {
+				if ! hasq \${FUNCNAME[1]} eautoreconf eaclocal _elibtoolize \\
+					eautoheader eautoconf eautomake autotools_run_tool \\
+					autotools_check_macro autotools_get_subdirs \\
+					autotools_get_auxdir ; then
+					eqawarn \"QA Notice: '${BIN}' called by \${FUNCNAME[1]}: \${CATEGORY}/\${PF}\"
+					eqawarn \"Use autotools.eclass instead of calling '${BIN}' directly.\"
+				fi
+			${BODY}
+			}"
 		else
-			eqawarn \"QA Notice: '${BIN}' called in global scope: \${CATEGORY}/\${PF}\"
+			FUNC_SRC="${BIN}() {
+				eqawarn \"QA Notice: '${BIN}' called by \${FUNCNAME[1]}: \${CATEGORY}/\${PF}\"
+			${BODY}
+			}"
 		fi
-		${BODY}
-		}";
 		eval "$FUNC_SRC" || echo "error creating QA interceptor ${BIN}" >&2
 	done
 	unset BIN_PATH BIN BODY FUNC_SRC
@@ -1686,23 +1720,12 @@ if ! hasq ${EBUILD_PHASE} clean && \
 	PDEPEND="${PDEPEND} ${E_PDEPEND}"
 
 	unset ECLASS E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
-
-	if [ "${EBUILD_PHASE}" != "depend" ] ; then
-		# Make IUSE defaults backward compatible with all the old shell code.
-		iuse_temp=""
-		for x in ${IUSE} ; do
-			if [[ ${x} == +* ]] || [[ ${x} == -* ]] ; then
-				iuse_temp="${iuse_temp} ${x:1}"
-			else
-				iuse_temp="${iuse_temp} ${x}"
-			fi
-		done
-		export IUSE=${iuse_temp}
-		unset x iuse_temp
-	fi
 	set +f
 fi
 
+# enable bashrc support for the clean phase
+[[ ${EBUILD_PHASE} == clean ]] && source_all_bashrcs
+
 # unset USE_EXPAND variables that contain only the special "*" token
 for x in ${USE_EXPAND} ; do
 	[ "${!x}" == "*" ] && unset ${x}
@@ -1717,12 +1740,6 @@ fi
 #a reasonable default for $S
 [[ -z ${S} ]] && export S=${WORKDIR}/${P}
 
-#wipe the interceptors.  we don't want saved.
-if hasq "depend" "${EBUILD_SH_ARGS}"; then
-	unset -f $QA_INTERCEPTORS
-	unset QA_INTERCEPTORS
-fi
-
 #some users have $TMP/$TMPDIR to a custom dir in their home ...
 #this will cause sandbox errors with some ./configure
 #scripts, so set it to $T.
@@ -1754,6 +1771,16 @@ if [ -n "${EBUILD_SH_ARGS}" ] ; then
 			ebuild_phase_with_hooks pkg_${EBUILD_SH_ARGS}
 			set +x
 		fi
+		if [[ $EBUILD_PHASE == postinst ]] && [[ -n $PORTAGE_UPDATE_ENV ]]; then
+			# Update environment.bz2 in case installation phases
+			# need to pass some variables to uninstallation phases.
+			(
+				unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
+				save_ebuild_env | filter_readonly_variables \
+					--filter-sandbox --allow-extra-vars | \
+					bzip2 -c -f9 > "$PORTAGE_UPDATE_ENV"
+			)
+		fi
 		;;
 	unpack|compile|test|clean|install)
 		if [ "${SANDBOX_DISABLED="0"}" == "0" ]; then
