@@ -1,12 +1,11 @@
 # Copyright 1999-2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: isolated-functions.sh 9148 2008-01-05 22:01:57Z zmedico $
+# $Id: isolated-functions.sh 10197 2008-05-05 06:03:43Z zmedico $
 
 # We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
 shopt -s expand_aliases
-alias die='diefunc "$FUNCNAME" "$LINENO" "$?"'
-alias assert='_pipestatus="${PIPESTATUS[*]}"; [[ "${_pipestatus// /}" -eq 0 ]] || diefunc "$FUNCNAME" "$LINENO" "$_pipestatus"'
+alias assert='_pipestatus="${PIPESTATUS[*]}"; [[ "${_pipestatus// /}" -eq 0 ]] || die'
 alias save_IFS='[ "${IFS:-unset}" != "unset" ] && old_IFS="${IFS}"'
 alias restore_IFS='if [ "${old_IFS:-unset}" != "unset" ]; then IFS="${old_IFS}"; unset old_IFS; else unset IFS; fi'
 
@@ -16,7 +15,7 @@ shopt -s extdebug
 #            [whitespacing for filenames],
 #            [whitespacing for line numbers])
 dump_trace() {
-	local funcname="" sourcefile="" lineno="" n e s="yes"
+	local funcname="" sourcefile="" lineno="" s="yes" n p
 	declare -i strip=${1:-1}
 	local filespacing=$2 linespacing=$3
 
@@ -24,14 +23,19 @@ dump_trace() {
 	# that the user will not be interested in. Therefore, the stack trace
 	# should only show calls that come after qa_call().
 	(( n = ${#FUNCNAME[@]} - 1 ))
+	(( p = ${#BASH_ARGV[@]} ))
 	while (( n > 0 )) ; do
 		[ "${FUNCNAME[${n}]}" == "qa_call" ] && break
+		(( p -= ${BASH_ARGC[${n}]} ))
 		(( n-- ))
 	done
-	(( n == 0 )) && (( n = ${#FUNCNAME[@]} - 1 ))
+	if (( n == 0 )) ; then
+		(( n = ${#FUNCNAME[@]} - 1 ))
+		(( p = ${#BASH_ARGV[@]} ))
+	fi
 
 	eerror "Call stack:"
-	for (( p = ${#BASH_ARGV[@]} ; n > ${strip} ; n-- )) ; do
+	while (( n > ${strip} )) ; do
 		funcname=${FUNCNAME[${n} - 1]}
 		sourcefile=$(basename ${BASH_SOURCE[${n}]})
 		lineno=${BASH_LINENO[${n} - 1]}
@@ -45,12 +49,11 @@ dump_trace() {
 			(( p -= ${BASH_ARGC[${n} - 1]} ))
 		fi
 		eerror "  $(printf "%${filespacing}s" "${sourcefile}"), line $(printf "%${linespacing}s" "${lineno}"):  Called ${funcname}${args:+ ${args}}"
+		(( n-- ))
 	done
 }
 
-diefunc() {
-	local funcname="$1" lineno="$2" exitcode="$3"
-	shift 3
+die() {
 	if [ -n "${QA_INTERCEPTORS}" ] ; then
 		# die was called from inside inherit. We need to clean up
 		# QA_INTERCEPTORS since sed is called below.
@@ -139,6 +142,12 @@ diefunc() {
 	exit 1
 }
 
+# We need to implement diefunc() since environment.bz2 files contain
+# calls to it (due to alias expansion).
+diefunc() {
+	die "${@}"
+}
+
 quiet_mode() {
 	[[ ${PORTAGE_QUIET} -eq 1 ]]
 }
@@ -161,7 +170,7 @@ elog_base() {
 			return 1
 			;;
 	esac
-	echo -e "$*" >> "${T}/logging/${EBUILD_PHASE:-other}.${messagetype}"
+	echo -ne "${messagetype} $*\n\0" >> "${T}/logging/${EBUILD_PHASE:-other}"
 	return 0
 }
 
@@ -437,7 +446,7 @@ save_ebuild_env() {
 		# There's no need to bloat environment.bz2 with internally defined
 		# functions and variables, so filter them out if possible.
 
-		unset -f dump_trace diefunc quiet_mode vecho elog_base eqawarn elog \
+		unset -f dump_trace die diefunc quiet_mode vecho elog_base eqawarn elog \
 			esyslog einfo einfon ewarn eerror ebegin _eend eend KV_major \
 			KV_minor KV_micro KV_to_int get_KV unset_colors set_colors has \
 			hasv hasq qa_source qa_call addread addwrite adddeny addpredict \
@@ -449,9 +458,10 @@ save_ebuild_env() {
 			abort_test abort_install dyn_compile dyn_test dyn_install \
 			dyn_preinst dyn_help debug-print debug-print-function \
 			debug-print-section inherit EXPORT_FUNCTIONS newdepend newrdepend \
-			newpdepend do_newdepend remove_path_entry killparent \
+			newpdepend do_newdepend remove_path_entry \
 			save_ebuild_env filter_readonly_variables preprocess_ebuild_env \
-			source_all_bashrcs ebuild_phase ebuild_phase_with_hooks
+			source_all_bashrcs ebuild_phase ebuild_phase_with_hooks \
+			${QA_INTERCEPTORS}
 
 		# portage config variables and variables set directly by portage
 		unset BAD BRACKET BUILD_PREFIX COLS \
@@ -462,11 +472,12 @@ save_ebuild_env() {
 			LAST_E_CMD LAST_E_LEN LD_PRELOAD MISC_FUNCTIONS_ARGS MOPREFIX \
 			NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
 			PORTAGE_ACTUAL_DISTDIR PORTAGE_ARCHLIST PORTAGE_BASHRC \
-			PORTAGE_BINPKG_TMPFILE PORTAGE_BUILDDIR \
+			PORTAGE_BINPKG_TAR_OPTS PORTAGE_BINPKG_TMPFILE PORTAGE_BUILDDIR \
 			PORTAGE_COLORMAP PORTAGE_CONFIGROOT PORTAGE_DEBUG \
 			PORTAGE_DEPCACHEDIR PORTAGE_GID PORTAGE_INST_GID \
 			PORTAGE_INST_UID PORTAGE_LOG_FILE PORTAGE_MASTER_PID \
-			PORTAGE_REPO_NAME PORTAGE_RESTRICT PORTAGE_WORKDIR_MODE PORTDIR \
+			PORTAGE_REPO_NAME PORTAGE_RESTRICT PORTAGE_UPDATE_ENV \
+			PORTAGE_WORKDIR_MODE PORTDIR \
 			PORTDIR_OVERLAY ${!PORTAGE_SANDBOX_*} PREROOTPATH \
 			PROFILE_PATHS PWORKDIR QA_INTERCEPTORS \
 			RC_DEFAULT_INDENT RC_DOT_PATTERN RC_ENDCOL \
@@ -474,6 +485,9 @@ save_ebuild_env() {
 			ROOT ROOTPATH RPMDIR STARTDIR TMP TMPDIR USE_EXPAND \
 			WARN XARGS _RC_GET_KV_CACHE
 
+		# user config variables
+		unset DOC_SYMLINKS_DIR INSTALL_MASK PKG_INSTALL_MASK
+
 		set
 		export
 	)
