@@ -374,10 +374,11 @@ install_mask() {
 		set +o noglob
 		quiet_mode || einfo "Removing ${no_inst}"
 		# normal stuff
-		rm -Rf ${root}/${no_inst} >&/dev/null
+		rm -Rf "${root}"/${no_inst} >&/dev/null
 
 		# we also need to handle globs (*.a, *.h, etc)
-		find "${root}" -path ${no_inst} -exec rm -fR {} \; >/dev/null
+		find "${root}" \( -path "${no_inst}" -or -name "${no_inst}" \) \
+			-exec rm -fR {} \; >/dev/null 2>&1
 	done
 	# set everything back the way we found it
 	set +o noglob
@@ -407,6 +408,11 @@ preinst_mask() {
 		 eerror "${FUNCNAME}: D is unset"
 		 return 1
 	fi
+
+	# Make sure $PWD is not ${D} so that we don't leave gmon.out files
+	# in there in case any tools were built with -pg in CFLAGS.
+	cd "${T}"
+
 	# remove man pages, info pages, docs if requested
 	for f in man info doc; do
 		if hasq no${f} $FEATURES; then
@@ -414,7 +420,7 @@ preinst_mask() {
 		fi
 	done
 
-	install_mask "${D}" ${INSTALL_MASK}
+	install_mask "${D}" "${INSTALL_MASK}"
 
 	# remove share dir if unnessesary
 	if hasq nodoc $FEATURES -o hasq noman $FEATURES -o hasq noinfo $FEATURES; then
@@ -464,27 +470,28 @@ preinst_suid_scan() {
 	fi
 	# total suid control.
 	if hasq suidctl $FEATURES; then
+		local sfconf
 		sfconf=${PORTAGE_CONFIGROOT}etc/portage/suidctl.conf
+		# sandbox prevents us from writing directly
+		# to files outside of the sandbox, but this
+		# can easly be bypassed using the addwrite() function
+		addwrite "${sfconf}"
 		vecho ">>> Performing suid scan in ${D}"
 		for i in $(find "${D}" -type f \( -perm -4000 -o -perm -2000 \) ); do
 			if [ -s "${sfconf}" ]; then
-				suid="$(grep "^${i/${D}}$" "${sfconf}")"
-				if [ "${suid}" = "${i/${D}}" ]; then
-					vecho "- ${i/${D}} is an approved suid file"
+				install_path=/${i#${D}}
+				if grep -q "^${install_path}\$" "${sfconf}" ; then
+					vecho "- ${install_path} is an approved suid file"
 				else
-					vecho ">>> Removing sbit on non registered ${i/${D}}"
+					vecho ">>> Removing sbit on non registered ${install_path}"
 					for x in 5 4 3 2 1 0; do echo -ne "\a"; sleep 0.25 ; done
 					vecho -ne "\a"
 					ls_ret=$(ls -ldh "${i}")
 					chmod ugo-s "${i}"
-					grep "^#${i/${D}}$" "${sfconf}" > /dev/null || {
-						# sandbox prevents us from writing directly
-						# to files outside of the sandbox, but this
-						# can easly be bypassed using the addwrite() function
-						addwrite "${sfconf}"
+					grep "^#${install_path}$" "${sfconf}" > /dev/null || {
 						vecho ">>> Appending commented out entry to ${sfconf} for ${PF}"
-						echo "## ${ls_ret%${D}*}${ls_ret#*${D}}" >> "${sfconf}"
-						echo "#${i/${D}}" >> "${sfconf}"
+						echo "## ${ls_ret%${D}*}${install_path}" >> "${sfconf}"
+						echo "#${install_path}" >> "${sfconf}"
 						# no delwrite() eh?
 						# delwrite ${sconf}
 					}
@@ -524,8 +531,10 @@ preinst_selinux_labels() {
 }
 
 dyn_package() {
-	cd "${PORTAGE_BUILDDIR}/image"
-	install_mask "${PORTAGE_BUILDDIR}/image" ${PKG_INSTALL_MASK}
+	# Make sure $PWD is not ${D} so that we don't leave gmon.out files
+	# in there in case any tools were built with -pg in CFLAGS.
+	cd "${T}"
+	install_mask "${PORTAGE_BUILDDIR}/image" "${PKG_INSTALL_MASK}"
 	local pkg_dest="${PKGDIR}/All/${PF}.tbz2"
 	local pkg_tmp="${PKGDIR}/All/${PF}.tbz2.$$"
 	local tar_options=""
@@ -533,9 +542,9 @@ dyn_package() {
 	# Sandbox is disabled in case the user wants to use a symlink
 	# for $PKGDIR and/or $PKGDIR/All.
 	export SANDBOX_ON="0"
-	tar ${tar_options} -cf - . | bzip2 -f > "${pkg_tmp}" || \
+	tar $tar_options -cf - $PORTAGE_BINPKG_TAR_OPTS -C "${D}" . | \
+		bzip2 -f > "${pkg_tmp}" || \
 		die "Failed to create tarball"
-	cd ..
 	export PYTHONPATH=${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}
 	python -c "import xpak; t=xpak.tbz2('${pkg_tmp}'); t.recompose('${PORTAGE_BUILDDIR}/build-info')"
 	if [ $? -ne 0 ]; then
@@ -561,7 +570,7 @@ dyn_package() {
 
 dyn_spec() {
 	tar czf "/usr/src/rpm/SOURCES/${PF}.tar.gz" \
-		"${O}/${PF}.ebuild" "${O}/files" || \
+		"${EBUILD}" "${FILESDIR}" || \
 		die "Failed to create base rpm tarball."
 
 	cat <<__END1__ > ${PF}.spec
@@ -597,7 +606,7 @@ dyn_rpm() {
 	addwrite /usr/src/rpm
 	addwrite "${RPMDIR}"
 	dyn_spec
-	rpmbuild -bb "${PF}.spec" || die "Failed to integrate rpm spec file"
+	rpmbuild -bb --clean --rmsource "${PF}.spec" || die "Failed to integrate rpm spec file"
 	install -D "/usr/src/rpm/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm" \
 		"${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm" || \
 		die "Failed to move rpm"
