@@ -1,9 +1,9 @@
 # elog_modules/mod_echo.py - elog dispatch module
 # Copyright 2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: mod_echo.py 7048 2007-06-27 01:25:02Z zmedico $
+# $Id: mod_echo.py 7675 2007-08-22 22:56:41Z zmedico $
 
-from output import EOutput
+from output import EOutput, colorize
 from portage_const import EBUILD_PHASES
 
 _items = []
@@ -11,7 +11,10 @@ def process(mysettings, key, logentries, fulltext):
 	global _items
 	_items.append((mysettings, key, logentries))
 
-def finalize():
+def finalize(mysettings=None):
+	"""The mysettings parameter is just for backward compatibility since
+	an older version of portage will import the module from a newer version
+	when it upgrades itself."""
 	global _items
 	printer = EOutput()
 	for mysettings, key, logentries in _items:
@@ -19,7 +22,8 @@ def finalize():
 		if mysettings["ROOT"] != "/":
 			root_msg = " merged to %s" % mysettings["ROOT"]
 		print
-		printer.einfo("Messages for package %s%s:" % (key, root_msg))
+		printer.einfo("Messages for package %s%s:" % \
+			(colorize("INFORM", key), root_msg))
 		print
 		for phase in EBUILD_PHASES:
 			if phase not in logentries:
