@@ -1,7 +1,7 @@
 # getbinpkg.py -- Portage binary-package helper functions
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: getbinpkg.py 7811 2007-09-26 04:00:51Z zmedico $
+# $Id: getbinpkg.py 9944 2008-04-22 18:30:54Z zmedico $
 
 if not hasattr(__builtins__, "set"):
 	from sets import Set as set
@@ -236,9 +236,6 @@ def make_http_request(conn, address, params={}, headers={}, dest=None):
 					break
 	
 	if (rc != 200) and (rc != 206):
-		sys.stderr.write(str(response.msg)+"\n")
-		sys.stderr.write(response.read()+"\n")
-		sys.stderr.write("address: "+address+"\n")
 		return None,rc,"Server did not respond successfully ("+str(response.status)+": "+str(response.reason)+")"
 
 	if dest:
@@ -310,7 +307,9 @@ def dir_get_list(baseurl,conn=None):
 			del page
 			listing = parser.get_anchors()
 		else:
-			raise Exception, "Unable to get listing: %s %s" % (rc,msg)
+			import portage_exception
+			raise portage_exception.PortageException(
+				"Unable to get listing: %s %s" % (rc,msg))
 	elif protocol in ["ftp"]:
 		if address[-1] == '/':
 			olddir = conn.pwd()
@@ -399,7 +398,12 @@ def file_get(baseurl,dest,conn=None,fcmd=None):
 	lexer = shlex.shlex(StringIO.StringIO(fcmd), posix=True)
 	lexer.whitespace_split = True
 	myfetch = [varexpand(x, mydict=variables) for x in lexer]
-	retval = spawn(myfetch, env=os.environ.copy())
+	fd_pipes= {
+		0:sys.stdin.fileno(),
+		1:sys.stdout.fileno(),
+		2:sys.stdout.fileno()
+	}
+	retval = spawn(myfetch, env=os.environ.copy(), fd_pipes=fd_pipes)
 	if retval != os.EX_OK:
 		sys.stderr.write("Fetcher exited with a failure condition.\n")
 		return 0
@@ -458,6 +462,8 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 	else:
 		keepconnection = 1
 
+	cache_path = "/var/cache/edb"
+
 	if makepickle is None:
 		makepickle = "/var/cache/edb/metadata.idx.most_recent"
 
@@ -485,7 +491,19 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 	if not metadata[baseurl].has_key("data"):
 		metadata[baseurl]["data"]={}
 
-	filelist = dir_get_list(baseurl, conn)
+	if not os.access(cache_path, os.W_OK):
+		sys.stderr.write("!!! Unable to write binary metadata to disk!\n")
+		sys.stderr.write("!!! Permission denied: '%s'\n" % cache_path)
+		return metadata[baseurl]["data"]
+
+	import portage_exception
+	try:
+		filelist = dir_get_list(baseurl, conn)
+	except portage_exception.PortageException, e:
+		sys.stderr.write("!!! Error connecting to '%s'.\n" % baseurl)
+		sys.stderr.write("!!! %s\n" % str(e))
+		del e
+		return metadata[baseurl]["data"]
 	tbz2list = match_in_array(filelist, suffix=".tbz2")
 	metalist = match_in_array(filelist, prefix="metadata.idx")
 	del filelist
@@ -602,6 +620,14 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 					# connection before sending a valid response".
 					conn, protocol, address, params, headers = create_conn(
 						baseurl)
+				except httplib.ResponseNotReady:
+					# With some http servers this error is known to be thrown
+					# from conn.getresponse() in make_http_request() when the
+					# remote file does not have appropriate read permissions.
+					# Maybe it's possible to recover from this exception in
+					# cases though, so retry.
+					conn, protocol, address, params, headers = create_conn(
+						baseurl)
 
 			if myid and myid[0]:
 				metadata[baseurl]["data"][x] = make_metadata_dict(myid)
