@@ -1,6 +1,6 @@
 # Copyright 1998-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: output.py 8548 2007-11-20 19:59:39Z zmedico $
+# $Id: output.py 10332 2008-05-13 23:11:50Z zmedico $
 
 __docformat__ = "epytext"
 
@@ -142,9 +142,12 @@ codes["INFORM"]                  = codes["darkgreen"]
 codes["UNMERGE_WARN"]            = codes["red"]
 codes["SECURITY_WARN"]           = codes["red"]
 codes["MERGE_LIST_PROGRESS"]     = codes["yellow"]
+codes["PKG_BLOCKER"]             = codes["red"]
+codes["PKG_BLOCKER_SATISFIED"]   = codes["darkblue"]
 codes["PKG_MERGE"]               = codes["darkgreen"]
 codes["PKG_MERGE_SYSTEM"]        = codes["darkgreen"]
 codes["PKG_MERGE_WORLD"]         = codes["green"]
+codes["PKG_UNINSTALL"]           = codes["red"]
 codes["PKG_NOMERGE"]             = codes["darkblue"]
 codes["PKG_NOMERGE_SYSTEM"]      = codes["darkblue"]
 codes["PKG_NOMERGE_WORLD"]       = codes["blue"]
@@ -237,7 +240,7 @@ def xtermTitle(mystr, raw=False):
 		if len(mystr) > max_len:
 			mystr = mystr[:max_len]
 		myt=os.environ["TERM"]
-		legal_terms = ["xterm","Eterm","aterm","rxvt","screen","kterm","rxvt-unicode","gnome"]
+		legal_terms = ["xterm","xterm-color","Eterm","aterm","rxvt","screen","kterm","rxvt-unicode","gnome","interix"]
 		for term in legal_terms:
 			if myt.startswith(term):
 				if not raw:
