@@ -1,7 +1,7 @@
 # portage: Constants
 # Copyright 1998-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_const.py 7932 2007-10-05 02:46:25Z zmedico $
+# $Id: portage_const.py 10199 2008-05-05 06:36:27Z zmedico $
 
 
 # ===========================================================================
@@ -23,6 +23,7 @@ CUSTOM_PROFILE_PATH     = USER_CONFIG_PATH+"/profile"
 PORTAGE_BASE_PATH       = os.path.join(os.sep, os.sep.join(__file__.split(os.sep)[:-2]))
 PORTAGE_BIN_PATH        = PORTAGE_BASE_PATH+"/bin"
 PORTAGE_PYM_PATH        = PORTAGE_BASE_PATH+"/pym"
+PORTAGE_PACKAGE_ATOM    = "sys-apps/portage"
 PROFILE_PATH            = "/etc/make.profile"
 LOCALE_DATA_PATH        = PORTAGE_BASE_PATH+"/locale"
 
@@ -51,15 +52,18 @@ REPO_NAME_LOC          = "profiles" + "/" + REPO_NAME_FILE
 INCREMENTALS = ["USE", "USE_EXPAND", "USE_EXPAND_HIDDEN", "FEATURES",
 	"ACCEPT_KEYWORDS", "ACCEPT_LICENSE",
 	"CONFIG_PROTECT_MASK", "CONFIG_PROTECT",
-	"PRELINK_PATH", "PRELINK_PATH_MASK"]
+	"PRELINK_PATH", "PRELINK_PATH_MASK", "PROFILE_ONLY_VARIABLES"]
 EBUILD_PHASES           = ["setup", "unpack", "compile", "test", "install",
-                          "preinst", "postinst", "prerm", "postrm", "other"]
+                          "package", "preinst", "postinst","prerm", "postrm",
+                          "other"]
 
 EAPI = 1
 
 HASHING_BLOCKSIZE        = 32768
 MANIFEST1_HASH_FUNCTIONS = ["MD5","SHA256","RMD160"]
 MANIFEST2_HASH_FUNCTIONS = ["SHA1","SHA256","RMD160"]
+
+MANIFEST1_REQUIRED_HASH = "MD5"
 MANIFEST2_REQUIRED_HASH = "SHA1"
 
 MANIFEST2_IDENTIFIERS = ["AUX","MISC","DIST","EBUILD"]
