@@ -1,7 +1,7 @@
 # deps.py -- Portage dependency resolution functions
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_dep.py 7730 2007-09-06 17:00:37Z zmedico $
+# $Id: portage_dep.py 9560 2008-03-28 12:20:03Z zmedico $
 
 
 # DEPEND SYNTAX:
@@ -44,10 +44,9 @@ def strip_empty(myarr):
 	@rtype: Array
 	@return: The array with empty elements removed
 	"""
-	for x in range(len(myarr)-1, -1, -1):
-		if not myarr[x]:
-			del myarr[x]
-	return myarr
+	return [x for x in myarr if x]
+
+_paren_whitespace_re = re.compile(r'\S(\(|\))|(\(|\))\S')
 
 def paren_reduce(mystr,tokenize=1):
 	"""
@@ -67,6 +66,12 @@ def paren_reduce(mystr,tokenize=1):
 	@rtype: Array
 	@return: The reduced string in an array
 	"""
+	global _dep_check_strict, _paren_whitespace_re
+	if _dep_check_strict:
+		m = _paren_whitespace_re.search(mystr)
+		if m is not None:
+			raise portage_exception.InvalidDependString(
+				"missing space by parenthesis: '%s'" % m.group(0))
 	mylist = []
 	while mystr:
 		left_paren = mystr.find("(")
@@ -364,7 +369,7 @@ def remove_slot(mydep):
 		mydep = mydep[:colon]
 	return mydep
 
-_invalid_atom_chars_regexp = re.compile("[()|?]")
+_invalid_atom_chars_regexp = re.compile("[()|?@]")
 
 def isvalidatom(atom, allow_blockers=False):
 	"""
@@ -394,7 +399,7 @@ def isvalidatom(atom, allow_blockers=False):
 		return 0
 	operator = get_operator(atom)
 	if operator:
-		if operator[0] in "<>" and atom[-1] == "*":
+		if operator[0] in "<>" and remove_slot(atom).endswith("*"):
 			return 0
 		if mycpv_cps and mycpv_cps[0] != "null":
 			# >=cat/pkg-1.0
