@@ -1,7 +1,7 @@
 # portage: Lock management code
 # Copyright 2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_locks.py 6932 2007-06-22 02:34:17Z zmedico $
+# $Id: portage_locks.py 9518 2008-03-27 04:50:32Z zmedico $
 
 
 import errno
@@ -79,7 +79,7 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0, waiting_msg=None):
 	except IOError, e:
 		if "errno" not in dir(e):
 			raise
-		if e.errno == errno.EAGAIN:
+		if e.errno in (errno.EACCES, errno.EAGAIN):
 			# resource temp unavailable; eg, someone beat us to the lock.
 			if waiting_msg is None:
 				if isinstance(mypath, int):
@@ -111,7 +111,7 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0, waiting_msg=None):
 
 		
 	if type(lockfilename) == types.StringType and \
-		myfd != HARDLINK_FD and os.fstat(myfd).st_nlink == 0:
+		myfd != HARDLINK_FD and _fstat_nlink(myfd) == 0:
 		# The file was deleted on us... Keep trying to make one...
 		os.close(myfd)
 		portage_util.writemsg("lockfile recurse\n",1)
@@ -122,6 +122,22 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0, waiting_msg=None):
 	portage_util.writemsg(str((lockfilename,myfd,unlinkfile))+"\n",1)
 	return (lockfilename,myfd,unlinkfile,locking_method)
 
+def _fstat_nlink(fd):
+	"""
+	@param fd: an open file descriptor
+	@type fd: Integer
+	@rtype: Integer
+	@return: the current number of hardlinks to the file
+	"""
+	try:
+		return os.fstat(fd).st_nlink
+	except EnvironmentError, e:
+		if e.errno == errno.ENOENT:
+			# Some filesystems such as CIFS return
+			# ENOENT which means st_nlink == 0.
+			return 0
+		raise
+
 def unlockfile(mytuple):
 	import fcntl
 
@@ -167,7 +183,7 @@ def unlockfile(mytuple):
 			# We won the lock, so there isn't competition for it.
 			# We can safely delete the file.
 			portage_util.writemsg("Got the lockfile...\n",1)
-			if os.fstat(myfd).st_nlink == 1:
+			if _fstat_nlink(myfd) == 1:
 				os.unlink(lockfilename)
 				portage_util.writemsg("Unlinked lockfile...\n",1)
 				locking_method(myfd,fcntl.LOCK_UN)
