@@ -101,7 +101,7 @@ class Manifest2Entry(ManifestEntry):
 class Manifest(object):
 	parsers = (parseManifest2, parseManifest1)
 	def __init__(self, pkgdir, distdir, fetchlist_dict=None,
-		manifest1_compat=True, from_scratch=False):
+		manifest1_compat=False, from_scratch=False):
 		""" create new Manifest instance for package in pkgdir
 		    and add compability entries for old portage versions if manifest1_compat == True.
 		    Do not parse Manifest file if from_scratch == True (only for internal use)
@@ -385,10 +385,8 @@ class Manifest(object):
 					else:
 						raise
 			if update_manifest:
-				fd = open(self.getFullname(), "w")
-				for myentry in myentries:
-					fd.write("%s\n" % str(myentry))
-				fd.close()
+				write_atomic(self.getFullname(),
+					"".join("%s\n" % str(myentry) for myentry in myentries))
 			if sign:
 				self.sign()
 		except (IOError, OSError), e:
@@ -425,7 +423,7 @@ class Manifest(object):
 		del self.fhashdict[ftype][fname]
 	
 	def hasFile(self, ftype, fname):
-		""" Return wether the Manifest contains an entry for the given type,filename pair """
+		""" Return whether the Manifest contains an entry for the given type,filename pair """
 		return (fname in self.fhashdict[ftype])
 	
 	def findFile(self, fname):
