@@ -1,6 +1,6 @@
 # Copyright 2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_util.py 7545 2007-08-02 17:19:32Z zmedico $
+# $Id: portage_util.py 10298 2008-05-12 00:01:26Z zmedico $
 
 from portage_exception import PortageException, FileNotFound, \
        OperationNotPermitted, PermissionDenied, ReadOnlyFileSystem
@@ -16,6 +16,11 @@ except ImportError:
 if not hasattr(__builtins__, "set"):
 	from sets import Set as set
 
+try:
+	import cStringIO as StringIO
+except ImportError:
+	import StringIO
+
 noiselimit = 0
 
 def writemsg(mystr,noiselevel=0,fd=None):
@@ -154,7 +159,7 @@ def stack_dicts(dicts, incremental=0, incrementals=[], ignore_none=0):
 		if final_dict is None:
 			final_dict = {}
 		for y in mydict.keys():
-			if mydict[y]:
+			if True:
 				if final_dict.has_key(y) and (incremental or (y in incrementals)):
 					final_dict[y] += " "+mydict[y][:]
 				else:
@@ -266,7 +271,9 @@ def grablines(myfilename,recursive=0):
 			myfile = open(myfilename, "r")
 			mylines = myfile.readlines()
 			myfile.close()
-		except IOError:
+		except IOError, e:
+			if e.errno == PermissionDenied.errno:
+				raise PermissionDenied(myfilename)
 			pass
 	return mylines
 
@@ -289,16 +296,41 @@ def writedict(mydict,myfilename,writekey=True):
 		return 0
 	return 1
 
+class _tolerant_shlex(shlex.shlex):
+	def sourcehook(self, newfile):
+		try:
+			return shlex.shlex.sourcehook(self, newfile)
+		except EnvironmentError, e:
+			writemsg("!!! Parse error in '%s': source command failed: %s\n" % \
+				(self.infile, str(e)), noiselevel=-1)
+			return (newfile, StringIO.StringIO())
+
 def getconfig(mycfg, tolerant=0, allow_sourcing=False, expand=True):
-	mykeys={}
+	if isinstance(expand, dict):
+		# Some existing variable definitions have been
+		# passed in, for use in substitutions.
+		expand_map = expand
+		expand = True
+	else:
+		expand_map = {}
+	mykeys = {}
 	try:
 		f=open(mycfg,'r')
 	except IOError, e:
+		if e.errno == PermissionDenied.errno:
+			raise PermissionDenied(mycfg)
 		if e.errno != errno.ENOENT:
 			raise
 		return None
 	try:
-		lex = shlex.shlex(f, posix=True)
+		if tolerant:
+			shlex_class = _tolerant_shlex
+		else:
+			shlex_class = shlex.shlex
+		# The default shlex.sourcehook() implementation
+		# only joins relative paths when the infile
+		# attribute is properly set.
+		lex = shlex_class(f, infile=mycfg, posix=True)
 		lex.wordchars=string.digits+string.letters+"~!@#$%*_\:;?,./-+{}"     
 		lex.quotes="\"'"
 		if allow_sourcing:
@@ -324,9 +356,9 @@ def getconfig(mycfg, tolerant=0, allow_sourcing=False, expand=True):
 				#invalid token
 				#lex.error_leader(self.filename,lex.lineno)
 				if not tolerant:
-					writemsg("!!! Invalid token (not \"=\") "+str(equ)+"\n",
-						noiselevel=-1)
-					raise Exception("ParseError: Invalid token (not '='): "+str(mycfg)+": line "+str(lex.lineno))
+					raise Exception("ParseError: Invalid token " + \
+						"'%s' (not '='): %s: line %s" % \
+						(equ, mycfg, lex.lineno))
 				else:
 					return mykeys
 			val=lex.get_token()
@@ -340,7 +372,8 @@ def getconfig(mycfg, tolerant=0, allow_sourcing=False, expand=True):
 				else:
 					return mykeys
 			if expand:
-				mykeys[key] = varexpand(val, mykeys)
+				mykeys[key] = varexpand(val, expand_map)
+				expand_map[key] = mykeys[key]
 			else:
 				mykeys[key] = val
 	except SystemExit, e:
@@ -1033,7 +1066,12 @@ def new_protect_filename(mydest, newmd5=None):
 	old_pfile = normalize_path(os.path.join(real_dirname, last_pfile))
 	if last_pfile and newmd5:
 		import portage_checksum
-		if portage_checksum.perform_md5(
-			os.path.join(real_dirname, last_pfile)) == newmd5:
-			return old_pfile
+		try:
+			last_pfile_md5 = portage_checksum.perform_md5(old_pfile)
+		except FileNotFound:
+			# The file suddenly disappeared or it's a broken symlink.
+			pass
+		else:
+			if last_pfile_md5 == newmd5:
+				return old_pfile
 	return new_pfile
